#ifndef _DISK_H
#define _DISK_H

typedef enum
{
	DISC_TYPE_GAMECUBE,
	DISC_TYPE_WII,
	DISC_TYPE_WII_DL,
	DISC_TYPE_UNKNOWN
} DISK_TYPE_ENUM;


typedef enum
{
	DISC_REGION_PAL,
	DISC_REGION_NTSC,
	DISC_REGION_JAPAN,
	DISC_REGION_UNKNOWN
} DISK_REGION_ENUM;

/*! \brief A structure that represents a Nintendo GameCube/Wii optical disc.
 */
typedef struct disc_tag
{
	DVD_DRIVE_TYPE *dvd;		//!< The structure for the DVD-drive the disc is inserted in.
	DISK_TYPE_ENUM type;		//!< The disc type.
	char system_id;				//!< A letter identifying the target system.
	char game_id[2 + 1];		//!< Two letters identifying the game.
	DISK_REGION_ENUM region;	//!< The disc region.
	char maker[3];				//!< Two letters identifying the maker of the game.
	U8 version;			        //!< A number identifying the game version.
	char *version_string;		//!< The same as <code>version</code>, in a more human-understandable format.
	char *title;				//!< The game title.
	U8 has_update;			    //!< True if the game contains a system update (Only possible for Wii discs).
	U32 sectors_no;			    //!< The number of sectors of the disc.

	/* Read function & stuff */
	S32 read_method;			//!< The read method ID.
    //!< The actual function that will be used to perform read operations, corresponding to <code>read_method</code>.
	S32 (*read_sector_fun_ptr) (struct disc_tag *d, U32 sector_no, U8 **data, U8 **rawdata);
    U8 unscrambling;			//!< If true, raw data read from the disc will be unscrambled to assure it is error-free. Disabling this is only useful for raw performance tests.
	
	/* Read cache */
	U32 cache_size;			//!< The number of blocks that will be cached when read.
	U8 **raw_cache;			//!< Memory area for raw sectors cache.
	U8 **cache;			    //!< Memory area for unscrambled sectors cache.
	U32 *cache_map;			//!< Data structure used by the caching system to know which blocks are in memory.
} DISK_TYPE;

extern DISK_TYPE *disc_new (char *dvd_device);
extern char *disc_get_drive_model_string (DISK_TYPE *d);
extern U8 disc_get_drive_support_status (DISK_TYPE *d);
extern U8 disc_init (DISK_TYPE *d);
extern S32 disc_read_sector (DISK_TYPE *d, U32 sector_no, U8 **data, U8 **rawdata);
extern char *disc_get_type (DISK_TYPE *d, DISK_TYPE_ENUM *dt, char **dt_s);
extern char *disc_get_gameid (DISK_TYPE *d, char **gid_s);
extern char *disc_get_region (DISK_TYPE *d, DISK_REGION_ENUM *dr, char **dr_s);
extern char *disc_get_maker (DISK_TYPE *d, char **m, char **m_s);
extern char *disc_get_version (DISK_TYPE *d, U8 *v, char **v_s);
extern char *disc_get_title (DISK_TYPE *d, char **t_s);
extern U32 disc_get_sectors_no (DISK_TYPE *d);
extern void *disc_destroy (DISK_TYPE *d);

#endif
