#include <stdio.h>
#include <windows.h>
#include "types.h"
#include "dvd_drive.h"
#include "disk.h"
#include "dumper.h"
#include "general.h"
#include "constants.h"

static void dumper_set_flushing (DUMPER_TYPE *dmp, U8 f);

DUMPER_TYPE *dumper_new (DISK_TYPE *d)
{
	DUMPER_TYPE *dmp;

	dmp = (DUMPER_TYPE *) malloc (sizeof (DUMPER_TYPE));
	memset (dmp, 0, sizeof (DUMPER_TYPE));
	dmp -> dsk = d;
	dumper_set_flushing (dmp, TRUE);

	return (dmp);
}

static void dumper_set_flushing (DUMPER_TYPE *dmp, U8 f)
{
	dmp -> flushing = f;
    #ifdef DEBUG_MODE
	fprintf (stderr,"Flushing %s", f ? "enabled" : "disabled");
    #endif
	return;
}

U8 dumper_set_iso_output_file (DUMPER_TYPE *dmp, char *outfile_iso)
{
	U8 out;
	FILE *fp;

	if (!outfile_iso)
    {
		/* Raw output disabled */
		out = TRUE;
		dmp -> start_sector_iso = -1;
		my_free (dmp -> outfile_iso);
		dmp -> outfile_iso = NULL;
	}
    else if (dmp -> outfile_iso)
    {
		printf("ISO output file already defined");
		out = FALSE;
	}
    else if (NULL == (fp = fopen(outfile_iso, "rb")))
    {		
		/* output file does not exist, start from scratch */
		out = TRUE;
		dmp -> start_sector_iso = 0;
		dmp -> outfile_iso = strdup(outfile_iso);
	}
    else  
    {
		/* Raw output file exists */
		fclose (fp);
		printf("ISO output file already exists, please delete old output file.\n");
		out = FALSE;
		dmp -> start_sector_iso = -1;
		free(dmp -> outfile_iso);
		dmp -> outfile_iso = NULL;
    }

	return (out);
}

void dumper_set_progress_callback (DUMPER_TYPE *dmp, progress_func progress, void *progress_data)
{
	dmp -> progress = progress;
	dmp -> progress_data = progress_data;

	return;
}

U8 dumper_prepare (DUMPER_TYPE *dmp)
{
	U8 out;
	U8 buf[RAW_SECTOR_SIZE];
	U32 i;
	
	if(dmp -> outfile_iso)
    {
		dmp -> start_sector = dmp -> start_sector_iso;
	} 
    else
    {
		MY_ASSERT (0);
	}

	/* Setup ISO output file */
	if(dmp -> outfile_iso)
    {
		dmp -> fp_iso = fopen (dmp -> outfile_iso, "wb");
		
        if(dmp -> fp_iso != NULL)
        {
			out = TRUE;
			#ifdef DEBUG_MODE
	        fprintf (stderr,"Writing to file \"%s\" in ISO format", dmp -> outfile_iso);
            #endif
		} else {
			out = FALSE;
			fclose (dmp -> fp_iso);
			dmp -> fp_iso = NULL;
		}
	}
    else
    {
		dmp -> fp_iso = NULL;
	}

	return (out);
}

S32 dumper_dump (DUMPER_TYPE *dmp)
{
	U8 out;
	U8 *rawbuf, *isobuf;
	U32 i, sectors_no;

	sectors_no = disc_get_sectors_no (dmp -> dsk);
	
    #ifdef DEBUG_MODE
	fprintf (stderr,"Starting dump process from sector %u...\n", dmp -> start_sector);
    #endif

	/* First call to progress function */
	if (dmp -> progress)
    {
        dmp -> progress (TRUE, dmp -> start_sector, sectors_no, dmp -> progress_data);
    }
		
		for (i = dmp -> start_sector, out = TRUE; i < sectors_no && out; i++)
        {
			if(FALSE == disc_read_sector(dmp -> dsk, i, &isobuf, &rawbuf))
            {
                /* got five retry errors in a row so drop out of dump */
                out = FALSE;
                break;
            }

			if(dmp -> fp_iso)
            {
				clearerr(dmp -> fp_iso);
				fwrite (isobuf, SECTOR_SIZE, 1, dmp -> fp_iso);

				if (ferror (dmp -> fp_iso))
                {
					printf("fwrite() to ISO output file failed");
					out = FALSE;
				}

				if (dmp -> flushing)
                {
					fflush (dmp -> fp_iso);
                }
			}

			if (dmp -> progress)
            {
                /* i + 1 'cause sectors range from 0 to N */
				dmp -> progress (FALSE, i + 1, sectors_no, dmp -> progress_data);
            }
		}

		if (dmp -> fp_iso)
        {
			fclose (dmp -> fp_iso);
        }

	return (out);
}

void *dumper_destroy (DUMPER_TYPE *dmp)
{
	free(dmp -> outfile_iso);
	free(dmp);

	return (NULL);
}
